unit NetCtrl;

{---------------------------------------------------------------------

(C) by Skelet (Georgi Georgiev) in terms of GNU General Public License

This is a small socketcall library. It depends only from kernel & rtl !

----------------------------------------------------------------------}

{$PACKRECORDS 1}

interface

const
 { domain constants: }
 AF_INET = 2; { Internet IP Protokol }
 AF_IPX  = 4; { Novell IPX }
 { socket types: }
 SOCK_STREAM = 1; { stream socket }
 SOCK_DGRAM  = 2; { datagram socket }
 SOCK_RAW    = 3;
 SOCK_RDM    = 4;
 SOCK_SEQPACKET = 5;
 SOCK_PACKET = 10;
 { protocol numbers for TCP/IP standart are in file /etc/protocols }

type
 PSockAddr = ^TSockAddr;
 TSockAddr = record
  Domain:word;
  Name:array[0..13] of char;
 end;
 TSockAddr_inet = record
  Domain:word;
  Port:word;
  Host:longint;
  Name:array[0..7] of char;
 end;
 TSockAddr_ipx = record
  Domain:word;
  Port:word;
  Host:longint;
  Node:array[0..5] of char;
  ipx_type:char;
  ipx_zero:char;
 end;

var
 NetError:longint;

function socket(domain, soc_type, protokol:longint):longint;
function bind(socket:longint; var SockAddr; SALen:longint):longint;
function connect(socket:longint; var SockAddr; SALen:longint):longint;
function listen(socket:longint; backlog:longint):longint;
function shutdown(socket:longint; how:longint):longint;
function accept(socket:longint; var SockAddr; var SockAddrLen:longint):longint;

function sendto(
  socket:longint;
  var Data; DataLen:longint;
  flags:longint;
  var SockAddr; SockAddrLen:longint):longint;

function recvfrom(
  socket:longint;
  var Data; DataLen:longint;
  flags:longint;
  var SockAddr; var SockAddrLen:longint):longint;

function getsockopt(
  socket:longint;
  level:longint; optname:longint;
  var OptVal; var OptLen:longint):longint;
function setsockopt(
  socket:longint;
  level:longint; optname:longint;
  var OptVal; OptLen:longint):longint;

function _fcntl(socket,cmd,value:longint):longint;

function inet_adr_to_num(name:string; var IP_addr:longint):boolean;
function inet_num_to_adr(IP_addr:longint):string;

implementation

{ $L /usr/lib/libc.a - not used now, all is done by SysCall ! }

uses Linux;

type
 TSocketArgs = array[0..5] of longint;

const
 SYS_Socket   = 1;
 SYS_Bind     = 2;
 SYS_Connect  = 3;
 SYS_Listen   = 4;
 SYS_Accept   = 5;
 SYS_GetSockName = 6;
 SYS_GetPeerName = 7;
 SYS_SocketPair = 8;
 SYS_Send     = 9;
 SYS_Recv     = 10;
 SYS_SendTo   = 11;
 SYS_RecvFrom = 12;
 SYS_ShutDown = 13;
 SYS_SetSockOpt = 14;
 SYS_GetSockOpt = 15;
 SYS_SendMsg  = 16;
 SYS_RecvMsg  = 17;

var Args:TSocketArgs;

function SocketCall(FnNum:longint):longint;
 var Regs:SysCallregs;
 begin
  Regs.reg2:=FnNum; Regs.reg3:=longint(@Args);
  SocketCall:=SysCall(SysCall_nr_socketcall,Regs);
  NetError:=ErrNo;
{ if SocketCall<0 then writeln('ErrNum=',NetError); }
{ Use upper line only for debug ! }
 end;

function socket(domain, soc_type, protokol:longint):longint;
 begin
  Args[0]:=domain; Args[1]:=soc_type; Args[2]:=protokol;
  socket:=SocketCall(Sys_Socket);
 end;
function bind(socket:longint; var SockAddr; SALen:longint):longint;
 begin
  Args[0]:=socket; Args[1]:=longint(@SockAddr); Args[2]:=SALen;
  bind:=SocketCall(Sys_Bind);
 end;
function connect(socket:longint; var SockAddr; SALen:longint):longint;
 begin
  Args[0]:=socket; Args[1]:=longint(@SockAddr); Args[2]:=SALen;
  connect:=SocketCall(Sys_Connect);
 end;
function listen(socket:longint; backlog:longint):longint;
 begin
  Args[0]:=socket; Args[1]:=backlog;
  listen:=SocketCall(Sys_Listen);
 end;
function shutdown(socket:longint; how:longint):longint;
 begin
  Args[0]:=socket; Args[1]:=how;
  shutdown:=SocketCall(Sys_ShutDown);
 end;
function accept(
 socket:longint; var SockAddr; var SockAddrLen:longint):longint;
 begin
  Args[0]:=socket; Args[1]:=longint(@SockAddr);
  Args[2]:=longint(@SockAddrLen);
  accept:=SocketCall(Sys_Accept);
 end;

function sendto(
 socket:longint; var Data; DataLen:longint; flags:longint;
 var SockAddr; SockAddrLen:longint):longint;
 begin
  Args[0]:=socket; Args[1]:=longint(@Data); Args[2]:=DataLen;
  Args[3]:=flags; Args[4]:=longint(@SockAddr); Args[5]:=SockAddrLen;
  sendto:=SocketCall(Sys_SendTo);
 end;

function recvfrom(
  socket:longint; var Data; DataLen:longint;
  flags:longint; var SockAddr; var SockAddrLen:longint):longint;
 begin
  Args[0]:=socket;
  Args[1]:=longint(@Data); Args[2]:=DataLen;
  Args[3]:=flags;
  Args[4]:=longint(@SockAddr); Args[5]:=longint(@SockAddrLen);
  recvfrom:=SocketCall(Sys_RecvFrom);
 end;

function getsockopt(
  socket:longint;
  level:longint; optname:longint;
  var OptVal; var OptLen:longint):longint;
 begin
  Args[0]:=socket;
  Args[1]:=level; Args[2]:=optname;
  Args[3]:=longint(@OptVal); Args[4]:=longint(@OptLen);
  getsockopt:=SocketCall(Sys_GetSockOpt);
 end;

function setsockopt(
  socket:longint;
  level:longint; optname:longint;
  var OptVal; OptLen:longint):longint;
 begin
  Args[0]:=socket;
  Args[1]:=level; Args[2]:=optname;
  Args[3]:=longint(@OptVal); Args[4]:=OptLen;
  setsockopt:=SocketCall(Sys_SetSockOpt);
 end;

function _fcntl(socket,cmd,value:longint):longint;
 var r:syscallregs;
 begin
  r.reg2:=socket; r.reg3:=cmd; r.reg4:=value;
  _fcntl:=SysCall(SysCall_nr_fcntl,r);
  NetError:=ErrNo;
  if _fcntl<0 then writeln('FCntlNum=',NetError);
 end;

type
 pchar = ^char;

function inet_adr_to_num(name:string; var IP_addr:longint):boolean;
 type TLong = array[0..3] of byte;
 var s0,s1:string; i:integer;
 begin
  inet_adr_to_num:=true; s0:=name; IP_addr:=0;
  if Pos('.',s0)>1 then begin
   s1:=copy(s0,1,Pos('.',s0)-1); delete(s0,1,Pos('.',s0));
   val(s1,TLong(IP_addr)[0],i);
   if Pos('.',s0)>1 then begin
    s1:=copy(s0,1,Pos('.',s0)-1); delete(s0,1,Pos('.',s0));
    val(s1,TLong(IP_addr)[1],i);
    if Pos('.',s0)>1 then begin
     s1:=copy(s0,1,Pos('.',s0)-1); delete(s0,1,Pos('.',s0));
     val(s1,TLong(IP_addr)[2],i);
     val(s0,TLong(IP_addr)[3],i);
    end else inet_adr_to_num:=false;
   end else inet_adr_to_num:=false;
  end else inet_adr_to_num:=false;
 end;

function inet_num_to_adr(IP_addr:longint):string;
 type TLong = array[0..3] of byte;
 var s0,s1,s2,s3:string[3];
 begin
  str(TLong(IP_addr)[0],s0);
  str(TLong(IP_addr)[1],s1);
  str(TLong(IP_addr)[2],s2);
  str(TLong(IP_addr)[3],s3);
  inet_num_to_adr:=s0+'.'+s1+'.'+s2+'.'+s3;
 end;

end.
