unit sr_recs;
interface

{$I options.pp}

uses
 {$IFDEF Turbo} Skl_defs, {$ENDIF}
 {$IFDEF FPC  } Sk_types, {$ENDIF}
 Date_lib;

const
{$IFDEF Cir}
 VLChar = '';
 RTChar = '';
 URChar = '';
 DTChar = '' {#208};
 SBChar = '' {#209};
{$ENDIF}
{$IFDEF Lat}
 VLChar = '';
 RTChar = '';
 URChar = '';
 SBChar = '';
{$ENDIF}

{------------- Field definitions: --------------}

const
 { Field Type constants: }
 CFList = 0; CFWord = 1; CFLong = 2; CFPntr = 3;
 CFReal = 4; CFStr  = 5; CFName = 6; CFDate = 7;
 CFTime = 8; CFMax  = CFTime;

{
    FType       Size        FPtr      StdLayout

    byte         1          ---       Lists, Boolean
    word         2          ---       Number16, Integer16
    Ptr          4          File#
    String       n          ---
    Name         n          ---
    Date         3          ---       DateStr
    Time         3          ---       TymeStr
    Real         6          ---       Money, Real
}

const
 FldMaxLen = 20;
 FldMaxCnt = 20;

const
 { Field flags constants: }
 ffIndexed = $01;
 ffUniTest = $02;
 ffRootInd = $04;
 ffFulShow = $08;
 ffCntSumm = $10;
 ffCntPntr = $20;

type
 TPeriod = record D0,D1:DateRec; end;
 PFldRec = ^TFldRec;
 TFldRec = record
  Used: byte; { 0 - active, $FF - erased }
  Name: array[0..FldMaxLen-1] of byte;
  FType: byte;
  Size: byte;
  FPtr: byte; { if FType=Ptr, number for pointed file }
  Flags:byte; { Indexed, UniqueTest, RootIndex, FullShow, CountSumm }
  FPName: SmallFName; { Name for pointed file }
  ShowSize:byte;      { column size in Brouse window }
  CntName:array[0..5] of byte; { links counter name }
 end;

procedure FldSetFlags(var F:TFldRec; aFlags:byte; Enable:boolean);
function  FldGetFlags(var F:TFldRec; aFlags:byte):boolean;
function FldLess(var F:TFldRec; var R1,R2):boolean;
function FldEqual(var F:TFldRec; var R1,R2):boolean;
function FldLimitStr(var F:TFldRec; var R):string;
procedure FldClear(var F:TFldRec; i:byte);
function  FldStdStrLen(var F:TFldRec):byte;
function  FldStdLinLen(var F:TFldRec):byte;
function  FldStdString(var F:TFldRec; var R; Depth:byte):string;
function  FldStdBrouseStr(var F:TFldRec; var R):string;
procedure FldStdClear(var F:TFldRec; var R);

function FieldStr(Fld:TFldRec):string;
function FldTStr(FT:byte):string;
function LoRecStr(var R; l:byte):string;

procedure LifeError(s:string);
procedure WarningMsg(s:string);

implementation

uses Str_def, sr_data;

type
 PString = ^string;

{-------------- Field definition: --------------}

function LoCaseB(b:byte):byte;
 const BigLSet = ['A'..'Z',''..''];
 begin LoCaseB:=b+(ord(chr(b) in BigLSet) shl 5) end;
function LoRecStr(var R; l:byte):string;
 var s:string; i:byte;
 begin
  s:=ClearSp(RecStr(R,l));
  if length(s)>0 then for i:=1 to Length(s)
   do s[i]:=chr(LoCaseB(ord(s[i])));
  LoRecStr:=s;
 end;

procedure FldSetFlags(var F:TFldRec; aFlags:byte; Enable:boolean);
 begin with F do if Enable
   then Flags:=Flags or aFlags
   else Flags:=Flags and (aFlags xor $FF)
 end;
function  FldGetFlags(var F:TFldRec; aFlags:byte):boolean;
 begin with F do FldGetFlags:=(Flags and aFlags)=aFlags end;
function FldLess(var F:TFldRec; var R1,R2):boolean;
 var
  b1:byte absolute R1;    b2:byte absolute R2;
  w1:word absolute R1;    w2:word absolute R2;
  l1:longint absolute R1; l2:longint absolute R2;
  rr1:real absolute R1;   rr2:real absolute R2;
  d1:DateRec absolute R1; d2:DateRec absolute R2;
  t1:TimeRec absolute R1; t2:TimeRec absolute R2;
  sr1:small absolute R1;  sr2:small absolute R2; i:byte; Eq:boolean;
 begin with F do case FType of
  CFList:FldLess:=b1<=b2;
  CFWord:FldLess:=w1<=w2;
  CFLong,CFPntr:FldLess:=l1<=l2;
  CFReal:FldLess:=rr1<=rr2;
  CFStr,CFName:begin
   Eq:=True; i:=0;
   while Eq and (i<Size)
    do begin Eq:=LoCaseB(sr1[i])=LoCaseB(sr2[i]); inc(i) end;
   dec(i);
   if not Eq then Eq:=LoCaseB(sr1[i])<LoCaseB(sr2[i]);
   FldLess:=Eq;
  end;
  CFDate:FldLess:=DateLess(d1,d2);
  CFTime:FldLess:=TimeLess(t1,t2);
 end end;
function FldEqual(var F:TFldRec; var R1,R2):boolean;
 var sr1:small absolute R1; sr2:small absolute R2; i:byte; Eq:boolean;
 begin with F do case FType of
  CFStr, CFName:begin
   Eq:=True; i:=0;
   while Eq and (i<Size) do
    begin Eq:=LoCaseB(sr1[i])=LoCaseB(sr2[i]); inc(i) end;
   FldEqual:=Eq;
  end;
  else FldEqual:=Equ(R1,R2,Size);
 end end;
function FldLimitStr(var F:TFldRec; var R):string;
 var ps:PString; sn:string[20];
  Pntr:longint absolute R;
  Period:TPeriod absolute R;
  Num:array[0..1] of longint absolute R;
 begin with F do begin
  New(ps); ps^:='';
  case FType of
   CFList,CFWord,CFLong:begin
           Str(Num[0],ps^);
           if Num[1]>Num[0]
{$IFDEF Cir}
            then begin Str(Num[1],sn); ps^:=' '+ps^+'..'+sn end
            else ps^:='  '+ps^;
{$ENDIF}
{$IFDEF Lat}
            then begin Str(Num[1],sn); ps^:='interval '+ps^+'..'+sn end
            else ps^:='equal to '+ps^;
{$ENDIF}
          end;
   CFPntr:ps^:=DFArr[FPtr]^.StdPtrLine(Pntr,3);
   CFDate:with Period do ps^:=DateStr_2(D0)+' .. '+DateStr_2(D1);
   CFStr,CFName:
{$IFDEF Cir}
          ps^:=': '+ClearSp(RecStr(R,Size))+'@';
{$ENDIF}
{$IFDEF Lat}
          ps^:='Pattern: '+ClearSp(RecStr(R,Size))+'@';
{$ENDIF}
  end;
  FldLimitStr:='('+ps^+')';
  Dispose(ps);
 end end;
procedure FldClear(var F:TFldRec; i:byte);
 var s:string[3];
 begin with F do begin
  Used:=0;  Str(i,s);
{$IFDEF Cir}
  StrRec('_'+s,Name,FldMaxLen);
{$ENDIF}
{$IFDEF Lat}
  StrRec('Field_'+s,Name,FldMaxLen);
{$ENDIF}
  FType:=CFStr; Size:=5; FPtr:=$FF;
  FPName:='';
  ShowSize:=FldStdStrLen(F);
  Flags:=0;
  Str(i,s);
{$IFDEF Cir}
  StrRec(''+s,CntName,6);
{$ENDIF}
{$IFDEF Lat}
  StrRec('Cnt_'+s,CntName,6);
{$ENDIF}
 end end;
function  FldStdStrLen(var F:TFldRec):byte;
 begin with F do case FType of
  CFList:FldStdStrLen:=5;
  CFWord:FldStdStrLen:=6;
  CFLong:FldStdStrLen:=11;
  CFPntr:FldStdStrLen:=15;
  CFReal:FldStdStrLen:=12;
  CFStr, CFName:FldStdStrLen:=Size;
  CFDate,CFTime:FldStdStrLen:=8;
 end end;
function  FldStdLinLen(var F:TFldRec):byte;
 begin with F do case FType of
  CFList:FldStdLinLen:=5;
  CFWord:FldStdLinLen:=7;
  CFLong:FldStdLinLen:=14;
  CFPntr:FldStdLinLen:=50;
  CFReal:FldStdLinLen:=14;
  CFStr, CFName:FldStdLinLen:=Size+2;
  CFDate:if ShowSize>8
   then FldStdLinLen:=10 else FldStdLinLen:=8;
  CFTime:FldStdLinLen:=8;
 end end;
function  FldStdString(var F:TFldRec; var R; Depth:byte):string;
 var ps:PString;
  b:byte absolute R;
  w:word absolute R;
  l:longint absolute R;
  rr:real absolute R;
  sr:small absolute R;
  d:DateRec absolute R;
  t:TimeRec absolute R;
 begin
  New(ps);
  with F do case FType of
   CFList:Str(b,ps^);
   CFWord:Str(w,ps^);
   CFLong:Str(l,ps^);
   CFPntr:if l>=0
          then ps^:=DFArr[FPtr]^.StdPtrLine(l,Depth)
          else ps^:='-';
   CFReal:Str(rr:12:2,ps^);
   CFStr, CFName:ps^:=RecStr(sr,Size);
   CFDate:case ShowSize of
            0,8:ps^:=DateStr_1(d);
           10:ps^:=DateStr_2(d);
          end;
   CFTime:ps^:=TimeStr_1(t);
  end;
  FldStdString:=ps^;
  Dispose(ps);
 end;
function  FldStdBrouseStr(var F:TFldRec; var R):string;
 var ps:PString;
 begin
  New(ps); ps^:=ClearSp(FldStdString(F,R,3));
  with F do case FType of
   CFPntr,CFStr,CFName:ps^:=JustStr(ps^,ShowSize);
   CFList,CFWord,CFLong,CFReal:if Length(ps^)>ShowSize
    then begin ps^:=''; LExpandStr(ps^,ShowSize,'#') end
    else LExpandStr(ps^,ShowSize,' ');
  end;
  FldStdBrouseStr:=ps^; Dispose(ps);
 end;
procedure FldStdClear(var F:TFldRec; var R);
 var
  b:byte absolute R;
  w:word absolute R;
  l:longint absolute R;
  rr:real absolute R;
  sr:small absolute R;
  d:DateRec absolute R;
  t:TimeRec absolute R;
 begin with F do case FType of
   CFList:b:=0;
   CFWord:w:=0;
   CFLong:l:=0;
   CFPntr:l:=-1;
   CFReal:rr:=0.0;
   CFStr, CFName:StrRec('',sr,Size);
   CFDate:DateGetD(d);
   CFTime:TimeClear(t);
 end end;

function FldTStr(FT:byte):string;
 begin case FT of
{$IFDEF Cir}
  CFList:FldTStr:=' (byte)';
  CFWord:FldTStr:=' (integer)';
  CFLong:FldTStr:=' (longint)';
  CFPntr:FldTStr:='';
  CFReal:FldTStr:=' (real)';
  CFStr :FldTStr:='';
  CFName:FldTStr:=' ';
  CFDate:FldTStr:='';
  CFTime:FldTStr:='';
{$ENDIF}
{$IFDEF Lat}
  CFList:FldTStr:='counter (byte)';
  CFWord:FldTStr:='counter (integer)';
  CFLong:FldTStr:='counter (longint)';
  CFPntr:FldTStr:='pointer';
  CFReal:FldTStr:='quontity (real)';
  CFStr :FldTStr:='string';
  CFName:FldTStr:='name';
  CFDate:FldTStr:='date';
  CFTime:FldTStr:='time';
{$ENDIF}
 end; end;

function FldTStr0(Fld:TFldRec):string;
 var s:string[15]; s1:string[3];
 begin
  case Fld.FType of
   CFList:s:='byte';
   CFWord:s:='integer';
   CFLong:s:='longint';
   CFPntr:s:='^['+DFArr[Fld.FPtr]^.LName+']';
   CFReal:s:='real';
{$IFDEF Cir}
   CFStr :s:='[';
   CFName:s:='[';
   CFDate:s:='';
   CFTime:s:='';
{$ENDIF}
{$IFDEF Lat}
   CFStr :s:='string[';
   CFName:s:='name[';
   CFDate:s:='date';
   CFTime:s:='time';
{$ENDIF}
  end;
  case Fld.FType of
   CFStr,CFName:begin Str(Fld.Size,s1); s:=s+s1+']' end;
  end;
  FldTStr0:=JustStr(s,15);
 end;

function FieldStr(Fld:TFldRec):string;
 var Opt,ShSz,FStr:String[9]; FW,i:byte;
 begin with Fld do begin
  FStr:=''; FW:=Flags;
  for i:=1 to 5 do begin
   if (FW and 1)>0 then FStr:=FStr+'x' else FStr:=FStr+' ';
   FW:=FW shr 1;
  end;
  Opt:=Blank(3); if FType=CFPntr then Str(FPtr:3,Opt);
  Str(ShowSize:3,ShSZ);
  FieldStr:=
   RecStr(Name,FldMaxLen-1)+VLChar+
   FldTStr0(Fld)+VLChar+
   ShSZ+' '+VLChar+
   FStr+VLChar+
   RecStr(CntName,6);
 end end;

{-----------------------------------------------}

procedure SomeMsg(s:string; fl:boolean);
 begin
 end;
procedure LifeError(s:string); begin SomeMsg(s,True) end;
procedure WarningMsg(s:string); begin SomeMsg(s,False) end;

{----------------------------------------------------------}

begin
 Writeln('SizeOf(TFldRec)=',SizeOf(TFldRec));
end.
